/*
  Calisto - An internet Talker
  Copyright (C) 1998-2001 Peter Howkins, Matthew Howkins and Simon Howkins

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,     
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License   
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Email: calisto@calisto.cx

  $Id: playerdb.c,v 1.13 2001/03/07 21:10:24 peter Exp $

 */
static const char rcsid[] = "$Id: playerdb.c,v 1.13 2001/03/07 21:10:24 peter Exp $";

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "config.h"
#include "globals.h"
#include "inifile2.h"
#include "library.h"
#ifdef USE_PLUGINS
#include "plugin2.h"
 #endif /* USE_PLUGINS */
#include "privs.h"
#include "strplus.h"
#include "structs.h"

bool validate_name(const char *n)
{
  /* this func can be changed as much as you want, except don't allow
     underscores as the first character of peoples names, as that would
     mess up the invitation system, I may use another character for 'banned'
     people sometime */

  if (strlen(n) < MIN_NAME_LENGTH)
    return FALSE;
  if (strlen(n) > MAX_NAME_LENGTH)
    return FALSE;
  if(!strisalnum(n)) 
    return FALSE;

  return TRUE;
}

bool player_exists(const char *n)
{
  char path[256], name[256];
  FILE *player;

  STRNCOPY(name, n, sizeof(name));
  strlower(name);
  STRNCOPY(path, PLAYER_DIR, sizeof(path));
  STRNAPPEND(path, name, sizeof(path));
  player = fopen(path, "r+");
  if(player == FALSE) {
    return FALSE;
  } else {
    fclose(player);
    return TRUE;
  }
}

int load_player(descriptor *d, const char *name) 
{
  char path[256];
  char n[MAX_NAME_LENGTH+1];
  character *c = &d->player;
  INIFILE *ini;
  FILE *f;
#ifdef USE_PLUGINS
  listnode *pNode = AllPlugins.head.next;
#endif /* USE_PLUGINS */

  /* inifile defaults */
  char def_name[] = "LOADFAIL";
  char def_pass[] = "";
  char def_prompt[] = "LOADFAIL>";
  unsigned long def_privs = 0;
  unsigned long def_prefs = 0;
  unsigned long def_rank = 0;
  unsigned long def_prevtime = 0;

  STRNCOPY(n, name, sizeof(n));
  strlower(n);

  STRNCOPY(path, PLAYER_DIR, sizeof(path));
  STRNAPPEND(path, n, sizeof(path));

  /* check to see if player exists */
  if((f = fopen(path, "r")) == NULL) {
    return 1;
  }
  fclose(f);

  if((ini = inifile_open(path)) == NULL) {
    log(debug, "Error occured loading player '%s'", name);
    return 1;
  }

  inifile_get_token(ini, "name",     "%s", c->name, def_name);
  inifile_get_token(ini, "password", "%s", c->password, def_pass);
  inifile_get_token(ini, "prompt",   "%s", c->prompt, def_prompt);
  inifile_get_token(ini, "privs",    "%lx", &c->privs, &def_privs);
  inifile_get_token(ini, "prefs",    "%lx", &c->prefs, &def_prefs);
  inifile_get_token(ini, "rank",     "%u",  &c->rank, &def_rank);
  inifile_get_token(ini, "prevtime", "%lu", &c->prevtime, &def_prevtime);

#ifdef USE_PLUGINS
  /* handle plugin loading */
  while (LIST_NODE_IS_REAL(pNode)) {
    struct plugin *p = LIST_GET_DATA(pNode, struct plugin *, pluginlink);
    void (*function)(INIFILE *, character *, void *);

    if((p->onload != NULL) && (p->offset != NO_PLUGIN_EXTRA_DATA)) {
      function = p->onload;
      function(ini, c, ((char *)d) + p->offset);
    }
    pNode = pNode->next;
  }
#endif /* USE_PLUGINS */

  inifile_close(ini, INIFILE_DONT_WRITE);  
  return 0;

}

int save_player(descriptor *d, const char *name)
{
  char path[256];
  char n[MAX_NAME_LENGTH+1];
  time_t now;
  unsigned long howlong;
  character *c = &d->player;
  INIFILE *ini;
#ifdef USE_PLUGINS
  listnode *pNode = AllPlugins.head.next;
#endif /* USE_PLUGINS */

  /* should we even be saving them ? */
  if(!haspriv(c, "n_save"))
    return 0;

  STRNCOPY(n, name, sizeof(n));
  strlower(n);

  if(talker_level == TALKER_INVITE || talker_level == TALKER_SU_INVITE) {
    STRNCOPY(path, "_", sizeof(path));
    STRNAPPEND(path, n, sizeof(path));
    if(player_exists(path) == TRUE) {
      STRNCOPY(path, PLAYER_DIR, sizeof(path));
      STRNAPPEND(path, "_", sizeof(path));
      STRNAPPEND(path, n, sizeof(path));
      remove(path);
    }
  }

  STRNCOPY(path, PLAYER_DIR, sizeof(path));
  STRNAPPEND(path, n, sizeof(path));

  if((ini = inifile_open(path)) == NULL) {
    log(debug, "Error occured loading player '%s'", name);
    return 1;
  }

  inifile_set_token(ini, "name",      "%s", c->name);
  inifile_set_token(ini, "password",  "%s", c->password);
  inifile_set_token(ini, "prompt",    "%s", c->prompt);
  inifile_set_token(ini, "privs",     "%lx", &c->privs);
  inifile_set_token(ini, "prefs",     "%lx", &c->prefs);
  inifile_set_token(ini, "rank",      "%u", &c->rank);
  now = time(NULL);
  howlong = (unsigned long) difftime(now, c->logintime);
  c->prevtime += howlong;
  inifile_set_token(ini, "prevtime",     "%lu", &c->prevtime);

#ifdef USE_PLUGINS
  /* handle plugin saving */
  while (LIST_NODE_IS_REAL(pNode)) {
    struct plugin *p = LIST_GET_DATA(pNode, struct plugin *, pluginlink);
    void (*function)(INIFILE *, character *, void *);

    if((p->onsave != NULL) && (p->offset != NO_PLUGIN_EXTRA_DATA)) {
      function = p->onsave;
      function(ini, c, ((char *)d) + p->offset);
    }
    pNode = pNode->next;
  }
#endif /* USE_PLUGINS */

  inifile_close(ini, INIFILE_DO_WRITE);  
  return 0;

}
