/*
  Calisto - An internet Talker
  Copyright (C) 1998-2001 Peter Howkins, Matthew Howkins and Simon Howkins

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,     
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License   
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Email: calisto@calisto.cx

  $Id: strplus.h,v 1.6 2001/03/07 21:09:06 peter Exp $

 */

#ifndef strplus_h
#define strplus_h

#include <string.h>

#ifndef BOOL
#define BOOL
typedef int bool;
#define FALSE  ((bool) 0)
#define TRUE   ((bool) 1)
#endif /* BOOL */

#define STREQ(s1, s2) (strcmp(s1, s2) == 0)
#define STRNEQ(s1, s2, n) (strncmp(s1, s2, n) == 0)
#define STRIEQ(s1, s2) (stricmp(s1, s2) == 0)
#define STRINEQ(s1, s2, n) (strnicmp(s1, s2, n) == 0) 

/* Copy ct to s. n is sizeof buffer at s. */
#define STRNCOPY(s, ct, n) ((void) ((n > 0) && \
                                    (strncpy(s, ct, n), (s)[(n) - 1] = '\0')))

/* Append ct to s. n is sizeof buffer at s. */
#define STRNAPPEND(s, ct, n) do { \
                               int len = strlen(s); \
                               if (len < n - 1) \
                                 strncat(s + len, ct, n - len - 1); \
                             } while (0)

/* portable strdup */
char *STRDUP(const char *s);

char *strlower(char *s);
char *strupper(char *s);

void strmunch(char *source, char **first, char **rest);

/* If the last character of the string is a \n, remove it, by replacing
   with \0. Do nothing if no \n found.
   Returns number of \n's chomped. (Can only be 0 or 1) */
int strchomp(char *s);

int stricmp(const char *s1, const char *s2);
int strnicmp(const char *s1, const char *s2, size_t n);

bool  strisalnum(const char *s);
bool  strisalpha(const char *s);
bool  striscntrl(const char *s);
bool  strisdigit(const char *s);
bool  strisgraph(const char *s);
bool  strislower(const char *s);
bool  strisprint(const char *s);
bool  strispunct(const char *s);
bool  strisspace(const char *s);
bool  strisupper(const char *s);
bool strisxdigit(const char *s);

char *strip_leading_white_space(const char *s);
void strip_trailing_white_space(char *s);

#endif /* strplus_h */
