/*
  Calisto - An internet Talker
  Copyright (C) 1998-2001 Peter Howkins, Matthew Howkins and Simon Howkins

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,     
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License   
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Email: calisto@calisto.cx

  $Id: structs.h,v 1.9 2001/03/07 21:09:50 peter Exp $

 */

#ifndef structs_h
#define structs_h

#include <time.h>

#include "netinet/in.h"

#include "dllist.h"

#ifndef BOOL
#define BOOL
typedef int bool;
#define FALSE ((bool) 0)
#define TRUE  ((bool) 1)
#endif /* BOOL */

#define MIN_NAME_LENGTH           1
#define MAX_NAME_LENGTH          12   /* Length of chars names */
#define MIN_PASSWORD_LENGTH       3
#define MAX_PASSWORD_LENGTH       8   /* Length of chars password */
#define CRYPT_PASSWORD_LENGTH    13   /* Length from crypt() function */
#define MAX_RAW_INPUT_BUFFER    512   /* Input buffer length */
#define MAX_HOST_NAME           256   /* host name len eg. fish.dripple.com */
#define MAX_TERMTYPE_LEN         20   /* terminal type eg xterm or vt100 */
#define MAX_PROMPT_LENGTH        12
#define OUTPUT_BUFFER          1024   /* Output buffer length */
#define MAX_GROUP_NAME_LENGTH    12

#define STATE_LOGIN     0
#define STATE_PASSWORD  1
#define STATE_PLAY      2
#define STATE_CLOSING   3
#define STATE_NEW1      4      
#define STATE_NEW2      5
#define STATE_NEW3      6

typedef struct character character;
typedef struct descriptor descriptor;
typedef struct logindata logindata;

/* used when making a new char to remember some things */
struct logindata {
  char name[MAX_NAME_LENGTH+1]; /* Chars name */
  char password[CRYPT_PASSWORD_LENGTH+1];    /* Chars password */
};

struct character {
  bool loggedin;
  char name[MAX_NAME_LENGTH+1];       /* Chars name */
  char password[CRYPT_PASSWORD_LENGTH+1];    /* Chars password */
  unsigned long privs;
  unsigned long prefs;
  unsigned rank;
  char prompt[MAX_PROMPT_LENGTH+1];
  char group[MAX_GROUP_NAME_LENGTH+1];/* name of their group */
  unsigned long prevtime;             /* num of seconds, prev to this login */

  time_t logintime;                   /* time they successfully logged in */

  listnode characterlink;             /* links together all active characters */
  listnode roomlink;                  /* links together the characters in each room */
};

struct descriptor {
  int sfd;                          /* socket descriptor */
  int state;                        /* the state the socket is in (eg playing
                                       or at password) */

  char inbuf[MAX_RAW_INPUT_BUFFER]; /* the raw data from the socket */
  int inbuf_used;                   /* number of bytes used in inbuf */
  int inbuf_used_lines;             /* number of bytes used for whole lines in inbuf */
  char outbuf[OUTPUT_BUFFER];
  time_t connecttime;               /* time connection was made */
  time_t idletime;                  /* time since last information received */

  char termtype[MAX_TERMTYPE_LEN+1];
  unsigned term_width;              /* the terminals width and height */
  unsigned term_height;

#ifdef IPV6
  struct in6_addr ip_addr;
#else
  struct in_addr ip_addr;             /* eg 127.0.0.1 */
#endif
  char hostname[MAX_HOST_NAME+1];     /* returned by gethostbyname */

  /* HACK - used in login process, has to be a nicer way though */
  logindata data;

  /* HACK - needed cos its tricky to find out if a socket is connected */
  bool safe;

  listnode descriptorlink;
  character player;                 /* The character using this socket */
};

#endif
